@@ -24,10 +24,10 @@ from bytestring_splitter import BytestringSplitter
 from constant_sorrow.constants import NO_KNOWN_NODES
 from collections import namedtuple, defaultdict
 from collections import OrderedDict
-from twisted.logger import Logger
 
 from .nicknames import Nickname
 from nucypher.crypto.api import keccak_digest
+from nucypher.utilities.logging import Logger
 
 
 class FleetSensor:
@@ -42,22 +42,27 @@ class FleetSensor:
     log = Logger("Learning")
     FleetState = namedtuple("FleetState", ("nickname", "icon", "nodes", "updated", "checksum"))
 
-    def __init__(self):
+    def __init__(self, domain: str):
+        self.domain = domain
         self.additional_nodes_to_track = []
         self.updated = maya.now()
         self._nodes = OrderedDict()
         self._marked = defaultdict(list)  # Beginning of bucketing.
         self.states = OrderedDict()
 
-    def __setitem__(self, key, value):
-        self._nodes[key] = value
+    def __setitem__(self, checksum_address, node_or_sprout):
+        if node_or_sprout.domain == self.domain:
+            self._nodes[checksum_address] = node_or_sprout
 
-        if self._tracking:
-            self.log.info("Updating fleet state after saving node {}".format(value))
-            self.record_fleet_state()
+            if self._tracking:
+                self.log.info("Updating fleet state after saving node {}".format(node_or_sprout))
+                self.record_fleet_state()
+        else:
+            msg = f"Rejected node {node_or_sprout} because its domain is '{node_or_sprout.domain}' but we're only tracking '{self.domain}'"
+            self.log.warn(msg)
 
-    def __getitem__(self, item):
-        return self._nodes[item]
+    def __getitem__(self, checksum_address):
+        return self._nodes[checksum_address]
 
     def __bool__(self):
         return bool(self._nodes)
@@ -77,6 +82,9 @@ class FleetSensor:
     def __repr__(self):
         return self._nodes.__repr__()
 
+    def population(self):
+        return len(self) + len(self.additional_nodes_to_track)
+
     @property
     def checksum(self):
         return self._checksum
