@@ -18,12 +18,26 @@ along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 import math
 import os
 import pprint
+import threading
 import time
 from typing import Callable, NamedTuple, Tuple, Union, Optional
+from typing import List
 from urllib.parse import urlparse
 
 import click
 import requests
+from eth_tester import EthereumTester
+from eth_tester.exceptions import TransactionFailed as TestTransactionFailed
+from eth_typing import ChecksumAddress
+from eth_utils import to_checksum_address
+from hexbytes.main import HexBytes
+from web3 import Web3, middleware, IPCProvider, WebsocketProvider, HTTPProvider
+from web3.contract import Contract, ContractConstructor, ContractFunction
+from web3.exceptions import ValidationError, TimeExhausted
+from web3.middleware import geth_poa_middleware
+from web3.providers import BaseProvider
+from web3.types import TxReceipt
+
 from constant_sorrow.constants import (
     INSUFFICIENT_ETH,
     NO_BLOCKCHAIN_CONNECTION,
@@ -32,37 +46,29 @@ from constant_sorrow.constants import (
     READ_ONLY_INTERFACE,
     UNKNOWN_TX_STATUS
 )
-from eth_tester.exceptions import TransactionFailed as TestTransactionFailed
-from eth_typing import ChecksumAddress
-from eth_utils import to_checksum_address
-from hexbytes.main import HexBytes
-from web3 import Web3, middleware
-from web3.contract import Contract, ContractConstructor, ContractFunction
-from web3.exceptions import TimeExhausted, ValidationError
-from web3.gas_strategies import time_based
-from web3.middleware import geth_poa_middleware
-from web3.providers import BaseProvider
-from web3.types import TxReceipt
-
 from nucypher.blockchain.eth.clients import EthereumClient, POA_CHAINS, InfuraClient
 from nucypher.blockchain.eth.decorators import validate_checksum_address
 from nucypher.blockchain.eth.providers import (
-    _get_auto_provider,
     _get_HTTP_provider,
     _get_IPC_provider,
+    _get_auto_provider,
     _get_mock_test_provider,
     _get_pyevm_test_provider,
     _get_test_geth_parity_provider,
     _get_websocket_provider
 )
 from nucypher.blockchain.eth.registry import BaseContractRegistry
-from nucypher.blockchain.eth.sol.compile import SolidityCompiler
+from nucypher.blockchain.eth.sol.compile.compile import multiversion_compile
+from nucypher.blockchain.eth.sol.compile.constants import SOLIDITY_SOURCE_ROOT
+from nucypher.blockchain.eth.sol.compile.types import SourceBundle
 from nucypher.blockchain.eth.utils import get_transaction_name, prettify_eth_amount
 from nucypher.characters.control.emitters import JSONRPCStdoutEmitter, StdoutEmitter
-from nucypher.utilities.datafeeds import datafeed_fallback_gas_price_strategy
 from nucypher.utilities.ethereum import encode_constructor_arguments
+from nucypher.utilities.gas_strategies import datafeed_fallback_gas_price_strategy, WEB3_GAS_STRATEGIES
 from nucypher.utilities.logging import GlobalLoggerSettings, Logger
 
+Web3Providers = Union[IPCProvider, WebsocketProvider, HTTPProvider, EthereumTester]  # TODO: Move to types.py
+
 
 class VersionedContract(Contract):
     version = None
@@ -77,16 +83,12 @@ class BlockchainInterface:
     TIMEOUT = 600  # seconds  # TODO: Correlate with the gas strategy - #2070
 
     DEFAULT_GAS_STRATEGY = 'fast'
-    GAS_STRATEGIES = {'glacial': time_based.glacial_gas_price_strategy,     # 24h
-                      'slow': time_based.slow_gas_price_strategy,           # 1h
-                      'medium': time_based.medium_gas_price_strategy,       # 5m
-                      'fast': time_based.fast_gas_price_strategy            # 60s
-                      }
+    GAS_STRATEGIES = WEB3_GAS_STRATEGIES
 
     process = NO_PROVIDER_PROCESS.bool_value(False)
-    Web3 = Web3
+    Web3 = Web3  # TODO: This is name-shadowing the actual Web3. Is this intentional?
 
-    _contract_factory = VersionedContract
+    _CONTRACT_FACTORY = VersionedContract
 
     class InterfaceError(Exception):
         pass
@@ -143,10 +145,15 @@ class BlockchainInterface:
 
         @property
         def insufficient_eth(self) -> str:
-            gas = (self.payload.get('gas', 1) * self.payload['gasPrice'])  # FIXME: If gas is not included...
-            cost = gas + self.payload.get('value', 0)
-            message = f'{self.payload} from {self.payload["from"][:8]} - {self.base_message}.' \
-                      f'Calculated cost is {cost} but sender only has {self.get_balance()}.'
+            try:
+                transaction_fee = self.payload['gas'] * self.payload['gasPrice']
+            except KeyError:
+                return self.default
+            else:
+                cost = transaction_fee + self.payload.get('value', 0)
+                message = f'{self.name} from {self.payload["from"][:8]} - {self.base_message}.' \
+                          f'Calculated cost is {prettify_eth_amount(cost)},' \
+                          f'but sender only has {prettify_eth_amount(self.get_balance())}.'
             return message
 
     def __init__(self,
@@ -156,7 +163,7 @@ class BlockchainInterface:
                  provider_process=NO_PROVIDER_PROCESS,
                  provider_uri: str = NO_BLOCKCHAIN_CONNECTION,
                  provider: BaseProvider = NO_BLOCKCHAIN_CONNECTION,
-                 gas_strategy: Union[str, Callable] = DEFAULT_GAS_STRATEGY):
+                 gas_strategy: Optional[Union[str, Callable]] = None):
 
         """
         TODO: #1502 - Move to API docs.
@@ -227,10 +234,10 @@ class BlockchainInterface:
         self._provider = provider
         self._provider_process = provider_process
         self.w3 = NO_BLOCKCHAIN_CONNECTION
-        self.client = NO_BLOCKCHAIN_CONNECTION         # type: EthereumClient
+        self.client = NO_BLOCKCHAIN_CONNECTION
         self.transacting_power = READ_ONLY_INTERFACE
         self.is_light = light
-        self.gas_strategy = self.get_gas_strategy(gas_strategy)
+        self.gas_strategy = gas_strategy or self.DEFAULT_GAS_STRATEGY
 
     def __repr__(self):
         r = '{name}({uri})'.format(name=self.__class__.__name__, uri=self.provider_uri)
@@ -279,20 +286,26 @@ class BlockchainInterface:
             self.log.debug('Injecting POA middleware at layer 0')
             self.client.inject_middleware(geth_poa_middleware, layer=0)
 
-        # Gas Price Strategy:
-        # Bundled web3 strategies are too expensive for Infura (it takes ~1 minute to get a price),
-        # so we use external gas price oracles, instead (see #2139)
-        if isinstance(self.client, InfuraClient):
+        self.client.add_middleware(middleware.time_based_cache_middleware)
+        # self.client.add_middleware(middleware.latest_block_based_cache_middleware)  # TODO: This line causes failed tests and nonce reuse in tests. See #2348.
+        self.client.add_middleware(middleware.simple_cache_middleware)
+
+        self.set_gas_strategy()
+
+    def set_gas_strategy(self, gas_strategy: Optional[Callable] = None):
+        if gas_strategy:
+            reported_gas_strategy = f"fixed/{gas_strategy.name}"
+        elif isinstance(self.client, InfuraClient):
             gas_strategy = datafeed_fallback_gas_price_strategy
+            self.gas_strategy = 'fast'
+            reported_gas_strategy = "datafeed/fast"
         else:
-            gas_strategy = self.gas_strategy
+            reported_gas_strategy = f"web3/{self.gas_strategy}"
+            gas_strategy = self.get_gas_strategy(self.gas_strategy)
         self.client.set_gas_strategy(gas_strategy=gas_strategy)
         gwei_gas_price = Web3.fromWei(self.client.gas_price_for_transaction(), 'gwei')
-        self.log.debug(f"Currently, our gas strategy returns a gas price of {gwei_gas_price} gwei")
-
-        self.client.add_middleware(middleware.time_based_cache_middleware)
-        # self.client.add_middleware(middleware.latest_block_based_cache_middleware)
-        self.client.add_middleware(middleware.simple_cache_middleware)
+        self.log.debug(f"Using gas strategy '{reported_gas_strategy}'. "
+                       f"Currently, it returns a gas price of {gwei_gas_price} gwei")
 
     def connect(self):
 
@@ -488,10 +501,12 @@ class BlockchainInterface:
                       sender_address: str,
                       payload: dict = None,
                       transaction_gas_limit: int = None,
+                      use_pending_nonce: bool = True,
                       ) -> dict:
 
+        nonce = self.client.get_transaction_count(account=sender_address, pending=use_pending_nonce)
         base_payload = {'chainId': int(self.client.chain_id),
-                        'nonce': self.client.w3.eth.getTransactionCount(sender_address, 'pending'),
+                        'nonce': nonce,
                         'from': sender_address}
 
         # Aggregate
@@ -510,6 +525,7 @@ class BlockchainInterface:
                                    payload: dict = None,
                                    transaction_gas_limit: Optional[int] = None,
                                    gas_estimation_multiplier: Optional[float] = None,
+                                   use_pending_nonce: Optional[bool] = None,
                                    ) -> dict:
 
         # Sanity checks for the gas estimation multiplier
@@ -522,7 +538,8 @@ class BlockchainInterface:
 
         payload = self.build_payload(sender_address=sender_address,
                                      payload=payload,
-                                     transaction_gas_limit=transaction_gas_limit)
+                                     transaction_gas_limit=transaction_gas_limit,
+                                     use_pending_nonce=use_pending_nonce)
         self.__log_transaction(transaction_dict=payload, contract_function=contract_function)
         try:
             transaction_dict = contract_function.buildTransaction(payload)  # Gas estimation occurs here
@@ -592,7 +609,6 @@ class BlockchainInterface:
         #
         # Broadcast
         #
-
         emitter.message(f'Broadcasting {transaction_name} Transaction ({cost} ETH @ {price_gwei} gwei)...',
                         color='yellow')
         try:
@@ -648,17 +664,24 @@ class BlockchainInterface:
                          transaction_gas_limit: Optional[int] = None,
                          gas_estimation_multiplier: Optional[float] = None,
                          confirmations: int = 0,
-                         fire_and_forget: bool = False  # do not wait for receipt.
-                         ) -> dict:
+                         fire_and_forget: bool = False,  # do not wait for receipt.  See #2385
+                         ) -> Union[TxReceipt, HexBytes]:
 
-        if fire_and_forget and confirmations > 0:
-            raise ValueError('Transaction Prevented: Cannot use confirmations and fire_and_forget options together.')
+        if fire_and_forget:
+            if confirmations > 0:
+                raise ValueError("Transaction Prevented: "
+                                 "Cannot use 'confirmations' and 'fire_and_forget' options together.")
+
+            use_pending_nonce = False  # TODO: #2385
+        else:
+            use_pending_nonce = None  # TODO: #2385
 
         transaction = self.build_contract_transaction(contract_function=contract_function,
                                                       sender_address=sender_address,
                                                       payload=payload,
                                                       transaction_gas_limit=transaction_gas_limit,
-                                                      gas_estimation_multiplier=gas_estimation_multiplier)
+                                                      gas_estimation_multiplier=gas_estimation_multiplier,
+                                                      use_pending_nonce=use_pending_nonce)
 
         # Get transaction name
         try:
@@ -699,7 +722,7 @@ class BlockchainInterface:
                 proxy_contract = self.client.w3.eth.contract(abi=proxy_abi,
                                                              address=proxy_address,
                                                              version=proxy_version,
-                                                             ContractFactoryClass=self._contract_factory)
+                                                             ContractFactoryClass=self._CONTRACT_FACTORY)
 
                 # Read this dispatcher's target address from the blockchain
                 proxy_live_target_address = proxy_contract.functions.target().call()
@@ -751,7 +774,7 @@ class BlockchainInterface:
         unified_contract = self.client.w3.eth.contract(abi=selected_abi,
                                                        address=selected_address,
                                                        version=selected_version,
-                                                       ContractFactoryClass=self._contract_factory)
+                                                       ContractFactoryClass=self._CONTRACT_FACTORY)
 
         return unified_contract
 
@@ -780,7 +803,15 @@ class BlockchainInterface:
 class BlockchainDeployerInterface(BlockchainInterface):
 
     TIMEOUT = 600  # seconds
-    _contract_factory = VersionedContract
+    _CONTRACT_FACTORY = VersionedContract
+
+    # TODO: Make more func - use as a parameter
+    # Source directories to (recursively) compile
+    SOURCES: List[SourceBundle] = [
+        SourceBundle(base_path=SOLIDITY_SOURCE_ROOT),
+    ]
+
+    _raw_contract_cache = NO_COMPILATION_PERFORMED
 
     class NoDeployerAddress(RuntimeError):
         pass
@@ -788,32 +819,15 @@ class BlockchainDeployerInterface(BlockchainInterface):
     class DeploymentFailed(RuntimeError):
         pass
 
-    def __init__(self,
-                 compiler: SolidityCompiler = None,
-                 ignore_solidity_check: bool = False,
-                 dry_run: bool = False,
-                 *args, **kwargs):
-
-        super().__init__(*args, **kwargs)
-        self.dry_run = dry_run
-        self.compiler = compiler or SolidityCompiler(ignore_solidity_check=ignore_solidity_check)
-
-    def connect(self):
+    def connect(self, compile_now: bool = True, ignore_solidity_check: bool = False) -> bool:
         super().connect()
-        self._setup_solidity(compiler=self.compiler)
-        return self.is_connected
-
-    def _setup_solidity(self, compiler: SolidityCompiler = None) -> None:
-        if self.dry_run:
-            self.log.info("Dry run is active, skipping solidity compile steps.")
-            return
-        if compiler:
+        if compile_now:
             # Execute the compilation if we're recompiling
             # Otherwise read compiled contract data from the registry.
-            _raw_contract_cache = compiler.compile()
-        else:
-            _raw_contract_cache = NO_COMPILATION_PERFORMED
-        self._raw_contract_cache = _raw_contract_cache
+            check = not ignore_solidity_check
+            compiled_contracts = multiversion_compile(source_bundles=self.SOURCES, compiler_version_check=check)
+            self._raw_contract_cache = compiled_contracts
+        return self.is_connected
 
     @validate_checksum_address
     def deploy_contract(self,
@@ -875,7 +889,7 @@ class BlockchainDeployerInterface(BlockchainInterface):
         contract = self.client.w3.eth.contract(address=address,
                                                abi=contract_factory.abi,
                                                version=contract_factory.version,
-                                               ContractFactoryClass=self._contract_factory)
+                                               ContractFactoryClass=self._CONTRACT_FACTORY)
 
         if enroll is True:
             registry.enroll(contract_name=contract_name,
@@ -900,9 +914,9 @@ class BlockchainDeployerInterface(BlockchainInterface):
             return requested_version, contract_data[requested_version]
         except KeyError:
             if requested_version != 'latest' and requested_version != 'earliest':
-                raise self.UnknownContract('Version {} of contract {} is not a locally compiled. '
-                                           'Available versions: {}'
-                                           .format(requested_version, contract_name, contract_data.keys()))
+                available = ', '.join(contract_data.keys())
+                raise self.UnknownContract(f'Version {contract_name} of contract {contract_name} is not a locally compiled. '
+                                           f'Available versions: {available}')
 
         if len(contract_data.keys()) == 1:
             return next(iter(contract_data.items()))
@@ -928,10 +942,10 @@ class BlockchainDeployerInterface(BlockchainInterface):
         """Retrieve compiled interface data from the cache and return web3 contract"""
         version, interface = self.find_raw_contract_data(contract_name, version)
         contract = self.client.w3.eth.contract(abi=interface['abi'],
-                                               bytecode=interface['bin'],
+                                               bytecode=interface['evm']['bytecode']['object'],
                                                version=version,
                                                address=address,
-                                               ContractFactoryClass=self._contract_factory)
+                                               ContractFactoryClass=self._CONTRACT_FACTORY)
         return contract
 
     def get_contract_instance(self,
@@ -960,7 +974,7 @@ class BlockchainDeployerInterface(BlockchainInterface):
         wrapped_contract = self.client.w3.eth.contract(abi=target_contract.abi,
                                                        address=wrapper_contract.address,
                                                        version=target_contract.version,
-                                                       ContractFactoryClass=self._contract_factory)
+                                                       ContractFactoryClass=self._CONTRACT_FACTORY)
         return wrapped_contract
 
     @validate_checksum_address
@@ -977,7 +991,7 @@ class BlockchainDeployerInterface(BlockchainInterface):
             proxy_contract = self.client.w3.eth.contract(abi=abi,
                                                          address=address,
                                                          version=version,
-                                                         ContractFactoryClass=self._contract_factory)
+                                                         ContractFactoryClass=self._CONTRACT_FACTORY)
 
             # Read this dispatchers target address from the blockchain
             proxy_live_target_address = proxy_contract.functions.target().call()
