@@ -72,7 +72,8 @@ def paint_stakes(emitter: StdoutEmitter,
         missing_info = f'Missing {missing} commitments{"s" if missing > 1 else ""}' if missing else f'Committed #{last_committed}'
 
     staker_data = [missing_info,
-                   f'{"Yes" if staker.is_restaking else "No"} ({"Locked" if staker.restaking_lock_enabled else "Unlocked"})',
+                   f'{"Yes" if staker.is_restaking else "No"} '
+                   f'({"Locked" if staker.restaking_lock_enabled else "Unlocked"})',
                    "Yes" if bool(staker.is_winding_down) else "No",
                    "Yes" if bool(staker.is_taking_snapshots) else "No",
                    pretty_fees,
@@ -96,8 +97,11 @@ def paint_stakes(emitter: StdoutEmitter,
                 emitter.echo(f"\t Blockchain Provider: {worker_data['blockchain_provider']}")
     emitter.echo(tabulate.tabulate(zip(STAKER_TABLE_COLUMNS, staker_data), floatfmt="fancy_grid"))
 
-    rows = list()
+    rows, inactive_substakes = list(), list()
     for index, stake in enumerate(stakes):
+        if stake.status().is_child(Stake.Status.INACTIVE):
+            inactive_substakes.append(index)
+
         if stake.status().is_child(Stake.Status.UNLOCKED) and not paint_unlocked:
             # This stake is unlocked.
             continue
@@ -108,6 +112,10 @@ def paint_stakes(emitter: StdoutEmitter,
 
     emitter.echo(tabulate.tabulate(rows, headers=STAKE_TABLE_COLUMNS, tablefmt="fancy_grid"))  # newline
 
+    if not paint_unlocked and inactive_substakes:
+        emitter.echo(f"Note that some sub-stakes are inactive: {inactive_substakes}\n"
+                     f"Run `nucypher stake list --all` to show all sub-stakes.", color='yellow')
+
 
 def prettify_stake(stake, index: int = None) -> str:
     start_datetime = stake.start_datetime.local_datetime().strftime("%b %d %H:%M %Z")
@@ -224,10 +232,10 @@ Global fee Range:
     emitter.echo(range_payload)
 
 
-def paint_min_rate(emitter, registry, policy_agent, staker_address):
-    paint_fee_rate_range(emitter, policy_agent)
-    minimum = policy_agent.min_fee_rate(staker_address)
-    raw_minimum = policy_agent.raw_min_fee_rate(staker_address)
+def paint_min_rate(emitter, staker):
+    paint_fee_rate_range(emitter, staker.policy_agent)
+    minimum = staker.min_fee_rate
+    raw_minimum = staker.raw_min_fee_rate
 
     rate_payload = f"""
 Minimum acceptable fee rate (set by staker for their associated worker):
